import { world, system } from '@minecraft/server'
import { Main } from './menus.js'

system.afterEvents.scriptEventReceive.subscribe(data => {
    const id = data.id
    const message = data.message
    const entity = data.sourceEntity;
    if(id === "gst:start") {
        entity.setProperty("gst:start", true);
        return;
    }
    if(id === "gst:reset") {
        if(!entity.isValid() || entity === undefined) return;
        entity.addTag("game_over");
        const start = entity.getProperty("gst:start");
        if(start === true) {
            entity.runCommandAsync("function gst/system/highscoreDetect");
        }
        if(entity.hasTag("player_1")) {
            entity.removeTag("player_1")
        }
        if(entity.hasTag("player_2")) {
            entity.removeTag("player_2")
        }
        if(entity.hasTag("player_3")) {
            entity.removeTag("player_3")
        }
        if(entity.hasTag("player_4")) {
            entity.removeTag("player_4")
        }
        system.runTimeout(() => {
            if(!entity.isValid() || entity === undefined) return;
            entity.teleport({ x: -23.5, y: -58, z: 11.5 },{ facingLocation: { x: -23.5, y: -58, z: 0.5 }});
            entity.setProperty("gst:start", false);
        },20);
    }
    if(id === "gst:open_menu") {
        if(message === "skins") {
            Main.Skins(entity);
        }
        if(message === "start") {
            Main.Start(entity);
        }
        if(message === "info") {
            Main.Info(entity);
        }
        else {
            return;
        }
    }
    if(id === "gst:spawn_fast_obstacle") {
        let random = 0
        random = 1 + (Math.floor(Math.random()*6))
        if(random === 1) {
            world.getDimension("overworld").spawnEntity(`gst:${message}_obstacle`, { x: 25, y: -43, z: -12.95});
        }
        if(random === 2) {
            world.getDimension("overworld").spawnEntity(`gst:${message}_obstacle`, { x: 25, y: -46, z: -12.95});
        }
        if(random === 3) {
            world.getDimension("overworld").spawnEntity(`gst:${message}_obstacle`, { x: 25, y: -49, z: -12.95});
        } else {
            return;
        }
        return;
    }
    if(id === "gst:give_coins") {
        const rewards = entity.getProperty("gst:rewards");
        const coinValue = entity.getProperty("gst:coins");
        const coinAdd = Number(message);
        entity.setProperty("gst:coins",coinValue + coinAdd);
        if(rewards < 5) {
            entity.setProperty("gst:rewards",rewards +1)
        } else {
            return;
        }
    }
    if(id === "gst:spawn_minecoin") {
        if(message === "100") {
            world.getDimension("overworld").spawnEntity("gst:minecoin", { x: 14, y: -43, z: -12.98});
            world.getDimension("overworld").spawnEntity("gst:minecoin", { x: 14, y: -46, z: -12.98});
            world.getDimension("overworld").spawnEntity("gst:minecoin", { x: 14, y: -49, z: -12.98});
            world.getDimension("overworld").spawnEntity("gst:minecoin", { x: 16, y: -43, z: -12.98});
            world.getDimension("overworld").spawnEntity("gst:minecoin", { x: 16, y: -46, z: -12.98});
            world.getDimension("overworld").spawnEntity("gst:minecoin", { x: 16, y: -49, z: -12.98});
            world.getDimension("overworld").spawnEntity("gst:minecoin", { x: 18, y: -43, z: -12.98});
            world.getDimension("overworld").spawnEntity("gst:minecoin", { x: 18, y: -46, z: -12.98});
            world.getDimension("overworld").spawnEntity("gst:minecoin", { x: 18, y: -49, z: -12.98});
            return;
        }
        let random = 0
        random = 1 + (Math.floor(Math.random()*4))
        if(random === 1) {
            world.getDimension("overworld").spawnEntity("gst:minecoin", { x: 25, y: -43, z: -13});
        }
        if(random === 2) {
            world.getDimension("overworld").spawnEntity("gst:minecoin", { x: 25, y: -46, z: -13});
        }
        if(random === 3) {
            world.getDimension("overworld").spawnEntity("gst:minecoin", { x: 25, y: -49, z: -13});
        }
        if(random === 4) {
            return;
        }
        return;
    }
    if(id === "gst:player_hit_obstacle") {
        if(!entity.isValid() || entity === undefined) return;
        const backgroundBlock = world.getDimension("overworld").getBlock({ x: -27, y: -60, z: -24 });
        if(entity.hasTag("onGround")) return;
        entity.addTag("onGround");
        entity.runCommandAsync("execute as @p[c=1,r=2] at @s run function gst/system/hit_obstacle");
        entity.applyKnockback(1,0,-0.75,0.3);
        if(backgroundBlock.permutation.matches("minecraft:gold_block")) {
            entity.runCommandAsync("camera @p[c=1,r=2] fade time 0 0 0.5 color 230 230 230");
        }
        system.runTimeout(() => {
            if(!entity.isValid() || !entity.hasTag("onGround")) return;
            entity.runCommandAsync("execute as @p at @s run scriptevent gst:reset");
        },20);
    }
    if(id === "gst:player_hit_coin") {
        system.run(() => {
            for(const player of world.getAllPlayers()) {
                const coinValue = player.getProperty("gst:coins");
                player.setProperty("gst:coins", coinValue +1);
                player.playSound("gst.coin");
            }
            entity.remove();
        });
        return;
    }
    if(id === "gst:update_name") {
        if(!entity.isValid() || entity === undefined) return;
        if(entity.typeId === "gst:jump_entity") {
            world.scoreboard.getObjective("gst:score").addScore(entity, 1);
            const score = world.scoreboard.getObjective("gst:score").getScore(entity);
            if(score !== undefined) {
                entity.nameTag = `§e${score}`
            } else {
                entity.nameTag = `§e0`
            }
        }
        return;
    }
    if(id === "gst:background_speed") {
        const speedValue = Number(message)
        if(entity.typeId.includes("background")) {
            entity.setProperty("gst:speed", speedValue);
        }
    }
});